package com.yihu.ehr.resolve.service.profile;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.ehr.fastdfs.FastDFSUtil;
import com.yihu.ehr.hbase.HBaseDao;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.csource.common.MyException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.List;
import java.util.Map;

@Service
public class PackageService {

    @Autowired
    private HBaseDao hbaseDao;
    @Autowired
    protected ObjectMapper objectMapper;
    @Autowired
    private FastDFSUtil fastDFSUtil;

    private static final String MasterTable = "HealthFile";

    private static final String SubTable = "HealthFileSub";


    /**
     *
     * 通过 主表的rowkey,删除hbase主细表数据,以及删除已经入库到fast dfs的文件
     * 该方法目前只对非结构化和影像档案有作用
     * @param profileId
     * @throws IOException
     */
    public void deleteFilePack(String profileId) throws IOException, MyException {
        Map<String, Object> obj = hbaseDao.getResultMap(MasterTable,profileId);
        if (obj != null) {
            if (obj.get("file_list") != null) {
                String file_list = (String) obj.get("file_list");
                List<Map<String, String>> list = objectMapper.readValue(file_list, List.class);
                //删除 fast dfs 中非结构化文档
                if(CollectionUtils.isNotEmpty(list)){
                    for(Map<String,String> map:list){
                        String url = map.get("url");
                        if(StringUtils.isNotBlank(url)){
                            int index = url.indexOf("/");
                            String groupName = url.substring(0,index);
                            String fileName = url.substring(index+1,url.length()-1);
                            //删除fastdfs档案包
                            fastDFSUtil.delete(groupName, fileName);
                        }
                    }
                }
            }
            if (obj.get("sub_rowkeys") != null) {
                String sub_rowkeys = (String) obj.get("sub_rowkeys");
                List<String> list = objectMapper.readValue(sub_rowkeys, List.class);
                //删除细表数据
                if(CollectionUtils.isNotEmpty(list)){
                    for(String subRow:list){
                        //删除主表档案
                        hbaseDao.delete(MasterTable,SubTable);
                    }
                }
            }
            //删除主表档案
            hbaseDao.delete(MasterTable,profileId);
        }
    }
}
