package com.yihu.ehr.resolve.service.resource.stage2;

import com.yihu.ehr.profile.ProfileType;
import com.yihu.ehr.profile.family.ResourceCells;
import com.yihu.ehr.resolve.config.TextSearchConfig;
import com.yihu.ehr.resolve.model.stage1.OriginalPackage;
import com.yihu.ehr.resolve.model.stage1.StandardPackage;
import com.yihu.ehr.resolve.model.stage2.ResourceBucket;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author progr1mmer.
 * @date Created on 2018/9/17.
 */
@Service
public class TextSearchService {

    @Autowired
    private TextSearchConfig textSearchConfig;
    @Autowired
    private RedisService redisService;

    /**
     * 生成检索字段
     * @param resourceBucket
     * @param originalPackage
     */
    public void generateSearchField(ResourceBucket resourceBucket, OriginalPackage originalPackage) {
        if (originalPackage.getProfileType() != ProfileType.Standard) {
            return;
        }
        StringBuilder stringBuilder = new StringBuilder();
        Map<String, String> master = textSearchConfig.getMaster();
        master.forEach((key, val) -> {
            String diagnosisName = "diagnosis_name";
            String chronicDiagnosis = "chronic_diagnosis";
            if (diagnosisName.equals(key)) {
                StandardPackage standardPackage = (StandardPackage)originalPackage;
                stringBuilder.append(val + ":" + StringUtils.join(standardPackage.getDiagnosisName(), ",") + ";");
            } else if (chronicDiagnosis.equals(key)) {
                StandardPackage standardPackage = (StandardPackage)originalPackage;
                Set<String> chronicNames = new HashSet<>();
                standardPackage.getDiagnosisCode().forEach(code -> {
                    String chronicInfo = redisService.getChronicInfo(code);
                    if (!StringUtils.isEmpty(chronicInfo)) {
                        String [] _chronicInfo = chronicInfo.split("-");
                        String flag = "0";
                        if (!flag.equals(_chronicInfo[1])) {
                            String healthProblem = redisService.getHpCodeByIcd10(code);
                            if (!StringUtils.isEmpty(healthProblem)) {
                                String separator = ";";
                                for (String hpCode : healthProblem.split(separator)) {
                                    String hpName = redisService.getHealthProblem(hpCode);
                                    if (StringUtils.isNotEmpty(hpName)) {
                                        chronicNames.add(hpName);
                                    }
                                }
                            }
                        }
                    }
                });
                stringBuilder.append(val + ":" + StringUtils.join(chronicNames, ",") + ";");
            } else {
                if (resourceBucket.getBasicRecord(key) != null) {
                    stringBuilder.append(val + ":" + resourceBucket.getBasicRecord(key) + ";");
                }
                if (resourceBucket.getMasterRecord().getDataGroup().get(key) != null) {
                    stringBuilder.append(val + ":" + resourceBucket.getMasterRecord().getDataGroup().get(key) + ";");
                }
            }
        });
        Map<String, String> sub = textSearchConfig.getSub();
        Map<String, Set<String>> datas = new HashMap<>(sub.size());
        resourceBucket.getSubRecords().forEach(subRecord ->
            sub.keySet().forEach(key -> {
                if (subRecord.getDataGroup().containsKey(key)) {
                    if (datas.containsKey(key)) {
                        datas.get(key).add(subRecord.getDataGroup().get(key));
                    } else {
                        Set<String> vals = new HashSet<>();
                        vals.add(subRecord.getDataGroup().get(key));
                        datas.put(key, vals);
                    }
                }
            })
        );
        sub.forEach((key, val) -> {
            if (datas.containsKey(key)) {
                stringBuilder.append(val + ":" + StringUtils.join(datas.get(key), ",") + ";");
            }
        });
        resourceBucket.insertBasicRecord(ResourceCells.SEARCH_FIELD, stringBuilder.toString());
    }
}